# Generated by roxygen2: do not edit by hand

S3method(as.dbMatrix,Matrix)
S3method(as.dbMatrix,default)
S3method(as.dbMatrix,dgCMatrix)
S3method(as.dbMatrix,dgTMatrix)
S3method(as.dbMatrix,matrix)
S3method(as.matrix,dbDenseMatrix)
S3method(as.matrix,dbMatrix)
S3method(as.matrix,dbSparseMatrix)
S3method(compute,dbDenseMatrix)
S3method(compute,dbMatrix)
S3method(compute,dbSparseMatrix)
export("colnames<-.dbMatrix")
export("rownames<-.dbMatrix")
export(as.dbMatrix)
export(colnames.dbMatrix)
export(dbDenseMatrix)
export(dbMatrix)
export(dbMatrix_from_tbl)
export(dbSparseMatrix)
export(db_svd)
export(ncol.dbMatrix)
export(nrow.dbMatrix)
export(rownames.dbMatrix)
export(sim_dbDenseMatrix)
export(sim_dbSparseMatrix)
export(to_named_ijx_tbl)
exportClasses(dbDenseMatrix)
exportClasses(dbMatrix)
exportClasses(dbSparseMatrix)
exportMethods("%in%")
exportMethods("[")
exportMethods("[<-")
exportMethods("dimnames<-")
exportMethods(Arith)
exportMethods(Math)
exportMethods(Ops)
exportMethods(Summary)
exportMethods(colMeans)
exportMethods(colSds)
exportMethods(colSums)
exportMethods(colVars)
exportMethods(dbLoad)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(head)
exportMethods(is.na)
exportMethods(length)
exportMethods(mean)
exportMethods(rowMeans)
exportMethods(rowSds)
exportMethods(rowSums)
exportMethods(rowVars)
exportMethods(t)
exportMethods(tail)
exportMethods(writeMM)
importClassesFrom(DBI,DBIConnection)
importClassesFrom(dbProject,dbData)
importFrom(Matrix,t)
importFrom(Matrix,writeMM)
importFrom(MatrixGenerics,colMeans)
importFrom(MatrixGenerics,colSds)
importFrom(MatrixGenerics,colSums)
importFrom(MatrixGenerics,colVars)
importFrom(MatrixGenerics,rowMeans)
importFrom(MatrixGenerics,rowSds)
importFrom(MatrixGenerics,rowSums)
importFrom(MatrixGenerics,rowVars)
importFrom(Rcpp,sourceCpp)
importFrom(data.table,":=")
importFrom(dbProject,"conn<-")
importFrom(dbProject,.check_con)
importFrom(dbProject,.check_name)
importFrom(dbProject,.check_overwrite)
importFrom(dbProject,.check_tbl)
importFrom(dbProject,conn)
importFrom(dbProject,dbList)
importFrom(dbProject,dbLoad)
importFrom(dbProject,dbReconnect)
importFrom(dbProject,to_view)
importFrom(dbProject,unique_table_name)
importFrom(dplyr,compute)
importFrom(methods,Arith)
importFrom(methods,Math)
importFrom(methods,Ops)
importFrom(methods,Summary)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,validObject)
importFrom(nanoarrow,as_nanoarrow_array_stream)
importFrom(rlang,abort)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(utils,capture.output)
useDynLib(dbMatrix, .registration = TRUE)
