
<!-- README.md is generated from README.Rmd. Please edit that file -->

# dbMatrix

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![CRAN
status](https://www.r-pkg.org/badges/version/dbMatrix)](https://CRAN.R-project.org/package=dbMatrix)

<!-- badges: end -->

`{dbMatrix}` is a core package in the
[`dbverse`](https://dbverse-org.github.io/dbverse/) library.

The goal of `{dbMatrix}` is to support larger-than-memory matrix
computations.

**Note: Work in progress! Bugs or unexpected behavior are likely to
occur. We welcome user feedback and reporting issues on the [Github
page](https://github.com/dbverse-org/dbmatrix-r/).**

## Installation

You can install the development version of dbMatrix from Github like so:

``` r
# install.packages("pak", repos = sprintf("https://r-lib.github.io/p/pak/stable/%s/%s/%s", .Platform$pkgType, R.Version()$os, R.Version()$arch))
pak::pak("dbverse-org/dbmatrix-r")
```
