# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.write_arrow_to_bpcells_cpp <- function(output_dir, n_rows, n_cols, col_scale, stream_factory) {
    .Call(`_dbMatrix_write_arrow_to_bpcells_cpp`, output_dir, n_rows, n_cols, col_scale, stream_factory)
}

.compute_op_svd_arrow_cpp <- function(stream_factory, n_rows, n_cols, row_offset, row_scale, col_scale, k) {
    .Call(`_dbMatrix_compute_op_svd_arrow_cpp`, stream_factory, n_rows, n_cols, row_offset, row_scale, col_scale, k)
}

