% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{get_tblName}
\alias{get_tblName}
\title{get_tblName}
\usage{
get_tblName(dbMatrix)
}
\arguments{
\item{dbMatrix}{A database-backed object inheriting from \code{dbData}.}
}
\value{
A character scalar giving the DuckDB table name associated with the
object.
}
\description{
get_tblName
}
\keyword{internal}
