% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operations.R
\name{is.na,dbMatrix-method}
\alias{is.na,dbMatrix-method}
\title{Element-wise is.na for dbMatrix}
\usage{
\S4method{is.na}{dbMatrix}(x)
}
\arguments{
\item{x}{A dbMatrix object.}
}
\value{
A dbMatrix with same dimensions, containing 1 where the original
value was NA and 0 otherwise.
}
\description{
Returns a dbMatrix with numeric values indicating NA positions (1 = NA, 0 = not NA).
}
\examples{
mat <- matrix(c(1, NA, 3, NA), nrow = 2)
dbmat <- as.dbMatrix(mat)
is.na(dbmat)

}
\concept{transform}
