% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dbMatrix.R
\name{compute.dbMatrix}
\alias{compute.dbMatrix}
\title{Force computation of a dbMatrix}
\usage{
\method{compute}{dbMatrix}(
  x,
  name = NULL,
  temporary = TRUE,
  dimnames = TRUE,
  overwrite = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{dbMatrix} object}

\item{name}{Name of the table to create. If NULL, a random name is generated.}

\item{temporary}{Logical. If TRUE (default), create a temporary table.}

\item{dimnames}{default = TRUE. If TRUE, the rownames and colnames will be
saved in the database. This allows full reconstruction of the dbMatrix object
using \code{\link[dbProject:dbLoad]{dbProject::dbLoad()}}.}

\item{overwrite}{Logical. If TRUE, overwrite the table if it already exists.
Default is FALSE.}

\item{...}{Additional arguments passed to methods (ignored).}
}
\value{
A \code{dbMatrix} object pointing to the new table.
}
\description{
Explicitly compute a dbMatrix and save it to a table in the database.
This overrides the default \code{dplyr::compute} to use a direct \verb{CREATE TABLE AS}
statement, which is more robust for large tables in DuckDB.
}
