% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{[,dbMatrix,dbIndex,missing,ANY-method}
\alias{[,dbMatrix,dbIndex,missing,ANY-method}
\alias{[,dbMatrix,missing,dbIndex,ANY-method}
\alias{[,dbMatrix,dbIndex,dbIndex,ANY-method}
\alias{[,dbMatrix,dbMatrix,missing,ANY-method}
\alias{[<-,dbMatrix,dbMatrix,missing,ANY-method}
\alias{[,dbMatrix,dbDenseMatrix,missing,ANY-method}
\alias{[,dbMatrix,missing,dbDenseMatrix,ANY-method}
\alias{[,dbMatrix,dbDenseMatrix,dbDenseMatrix,ANY-method}
\title{Extract or replace values in database-backed matrices}
\usage{
\S4method{[}{dbMatrix,dbIndex,missing,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{dbMatrix,missing,dbIndex,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{dbMatrix,dbIndex,dbIndex,ANY}(x, i, j, ..., drop = FALSE)

\S4method{[}{dbMatrix,dbMatrix,missing,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{dbMatrix,dbMatrix,missing,ANY}(x, i, j) <- value

\S4method{[}{dbMatrix,dbDenseMatrix,missing,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{dbMatrix,missing,dbDenseMatrix,ANY}(x, i, j, ..., drop = FALSE)

\S4method{[}{dbMatrix,dbDenseMatrix,dbDenseMatrix,ANY}(x, i, j, ..., drop = FALSE)
}
\arguments{
\item{x}{A \code{dbMatrix} object.}

\item{i}{Row, logical matrix, or matrix-style index.}

\item{j}{Column index.}

\item{...}{Additional arguments.}

\item{drop}{Ignored; included for matrix API compatibility.}

\item{value}{Replacement value.}
}
\value{
A subsetted or modified \code{dbMatrix}, or an extracted vector for
matrix-style indexing.
}
\description{
Methods for subsetting and replacing values in \code{dbMatrix}
objects.
}
\concept{dbMatrix}
